#######################################################
#   Project: OCR of Ottoman Printed Nasih Font        #
#   Osmanlica.com test data set comparison script     #
#   2021.04.01, Ishak DOLEK, ishakdolek54@gmail.com   #
#   Download: osmanlica.com/test                      #
#######################################################
import difflib
import sys
import re
import unicodedata

dir = ''
with open(dir + 'GroundTruth.txt', encoding="utf8") as f:
    grounds = [line for line in f]
with open(dir + 'Output.txt',  encoding="utf8") as f:
    ocrs    = [line for line in f]

normalizeWhitespace = 1
if(normalizeWhitespace):
    grounds = [re.sub(r"\s+", " ", line, flags=re.UNICODE).strip() for line in grounds]
    ocrs    = [re.sub(r"\s+", " ", line, flags=re.UNICODE).strip() for line in ocrs]   

normalizeSpecialCases = 1
fromChar = ["(", "ە", "\u200e","\u200f","\ufeff","\u200c"] # handle paranthesis problem & 2 diff He into single He
toChar   = [")", "ه", ""      ,""      ,""      ,""      ]
if(normalizeSpecialCases):
    for i in range(len(fromChar)):
        grounds = [x.replace(fromChar[i], toChar[i]) for x in grounds]
        ocrs    = [x.replace(fromChar[i], toChar[i]) for x in ocrs   ]   

normalizeLetters = 1
fr = "? ; , . ۽ﺀ آأإٱٲٳٵݳݴﭐﭑﺁﺂﺃﺄﺇﺈﺍﺎ ٮݕࢠࢡࢶﭒﭓﭔﭕﺏﺐﺑﺒ ݐݒݓݔࢷﭖﭗﭘﭙﭚﭛﭜﭝ ٹٺٻټٽٿڀݖࢸﭞﭟﭠﭡﭦﭧﭨﭩﺕﺖﺗﺘ ﭢﭣﭤﭥݑﺙﺚﺛﺜ ڃڄﭲﭳﭴﭵﭶﭷﭸﭹݼࢢﺝﺞﺟﺠ ڿݘﭺﭻﭼﭽﭾﭿﮀﮁچڇݯݮ ﺡﺢﺣﺤ ﺥﺦﺧﺨځڂݲڅݗ ﺩﺪډڊڍݚࢮﮂﮃ ڋڌڎڏڐۮݙﮄﮅﮆﮇﮈﮉڈﺫﺬ ﺭﺮړڔڕږݛࢪ ﺯﺰڑڒݫݬݱࢹﮌﮍۯڗڙࢲ ﮊﮋ ﺱﺲﺳﺴ ﺵﺶﺷﺸښڛڜۺݜݭݰݽݾ ﺹﺺﺻﺼ ﺽﺾﺿﻀڝڞۻࢯ ﻁﻂﻃﻄ ﻅﻆﻇﻈڟࢣ ﻉﻊﻋﻌࢳ ﻍﻎﻏﻐۼڠݝݞݟ ﻑﻒﻓﻔڡڢڣڤڥڦݠݡࢤࢻﭪﭫﭬﭭﭮﭯﭰﭱ ﻕﻖﻗﻘڨڧࢥٯࢥࢼ ﻙﻚﻛﻜکڪڮؼݤࢴﮎﮏﮐﮑ ڲڳڰࢰﮒﮓﮔﮕﮖﮗﮘﮙ ػګڬڱڴݢݣݿﮚﮛﮜﮝﯓﯔﯕﯖ ﻝﻞﻟﻠڵڶڷڸݪࢦ ﻡﻢﻣﻤ۾ݥݦࢧ ﻥﻦﻧﻨڹںڻڼڽݧݨݩࢽﮞﮟﮠﮡﮢﮣ ﻭﻮؤٶٷۄۅۆۇۈۉۊۋۏݸݹࢫࢱﯗﯘﯙﯚﯛﯜﯝﯞﯟﯠﯡﯢﯣﺅﺆ ھۿةہۂۃۿﮥﮧﮨﮩﮪﮫﮬﮭﺔﻬﻫﻪ ﻩۀﮤﮦﺓ ﻵﻶﻷﻸﻹﻺﻻﻼ ؠئؽؾؿٸيۑےۓیۍێېݵݶݷݺݻࢨࢩࢬࢺﮮﮯﮰﮱﯤﯥﯦﯧﯨﯩﯰﯱﯲﯳﯴﯵﯶﯷﯸﯹﯺﯻﯼﯽﯾﯿﯪﯫﯬﯭﯮﯯﺉﺊﺋﺌﻯﻰﻱﻲﻳﻴ ۰ ۱ ۲ ۳ ۴ ۵ ۶ ۷ ۸ ۹"
to = "؟؛،۔ءابپتثجچحخدذرزژسشصضطظعغفقكگڭلمنوهەﻻى٠١٢٣٤٥٦٧٨٩" 
subs = dict(zip(fr.split(" "), list(to)))
def normalize_arabic(text):
    for key, value in subs.items():
        text = re.sub(u"["+re.escape(key)+"]" , value, text)
    return text

def remove_accents(input_str):
    nfkd_form = unicodedata.normalize('NFKD', input_str)
    return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])

if(normalizeLetters):
    grounds = [normalize_arabic(remove_accents(line)) for line in grounds] # normalize letters & remove diacritics
    ocrs    = [normalize_arabic(remove_accents(line)) for line in ocrs]    # normalize letters & remove diacritics

grounds = " ".join(grounds)
ocrs    = " ".join(ocrs)   
sm = difflib.SequenceMatcher(None, grounds, ocrs, autojunk=False)

print("\n\nGround Truth:\n")
print(grounds)
print("\n\nOCR Output:\n")
print(ocrs)
print("\n SequenceMatcher.ratio(): "+str(sm.ratio()))
